﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using STSdb.Data;
using System.Reflection.Emit;
using System.Reflection;
using System.Threading;

namespace GettingStarted
{
    public partial class DataGridFrame : UserControl
    {
        private const int PAGE_SIZE = 100;
        private XTable table;
        private PropertyInfo[] properies;

        public DataGridFrame()
        {
            InitializeComponent();
        }

        private object FirstKey
        {
            get { return dataGridView1.Rows.Count > 0 ? dataGridView1[0, 0].Value : null; }
        }

        private object LastKey
        {
            get { return dataGridView1.Rows.Count > 0 ? dataGridView1[0, dataGridView1.Rows.Count - 1].Value : null; }
        }

        private void PrepareColumns()
        {
            dataGridView1.Rows.Clear();
            dataGridView1.Columns.Clear();
            if (table == null)
                return;

            Type keyType = table.KeyType;
            Type recordType = table.RecordType;

            dataGridView1.Columns.Add("_Key", "Key");
            dataGridView1.Columns[0].ValueType = keyType;
            dataGridView1.Columns[0].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;
            dataGridView1.Columns[0].ReadOnly = true;

            properies = recordType.GetProperties();
            foreach (var pi in properies)
            {
                int idx = dataGridView1.Columns.Add(pi.Name, pi.Name);
                dataGridView1.Columns[idx].ValueType = pi.PropertyType;
                dataGridView1.Columns[idx].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
            }
        }

        private object[] GetValues(Row<object, object> row)
        {
            object[] values = new object[1 + properies.Length];
            values[0] = row.Key;

            for (int i = 0; i < properies.Length; i++)
                values[i + 1] = properies[i].GetValue(row.Record, null);

            return values;
        }

        private void ShowData(object startKey, bool forward)
        {
            Cursor = Cursors.WaitCursor;
            try
            {
                toolStripLabel1.Text = "";
                dataGridView1.Rows.Clear();
                if (table == null)
                    return;

                if (forward)
                {
                    IEnumerable<Row<object, object>> enumerator = startKey != null ? table.Forward(startKey) : table.Forward();
                    foreach (var row in enumerator.Take(PAGE_SIZE))
                    {
                        object[] values = GetValues(row);
                        dataGridView1.Rows.Add(values);
                    }
                }
                else
                {
                    IEnumerable<Row<object, object>> enumerator = startKey != null ? table.Backward(startKey) : table.Backward();
                    foreach (var row in enumerator.Take(PAGE_SIZE))
                    {
                        object[] values = GetValues(row);
                        dataGridView1.Rows.Insert(0, values);
                    }
                }

                toolStripLabel1.Text = String.Format("{0} records", table.Count);
            }
            finally
            {
                Cursor = Cursors.Arrow;
            }
        }

        private void dataGridView1_CellValueChanged(object sender, DataGridViewCellEventArgs e)
        {
            object key = dataGridView1[0, e.RowIndex].Value;
            object value = dataGridView1[e.ColumnIndex, e.RowIndex].Value;

            object record = table[key];
            PropertyInfo property = properies[e.ColumnIndex - 1];
            property.SetValue(record, value, null);
            table[key] = record;
            table.Commit();
        }

        private void toolStripButtonBegin_Click(object sender, EventArgs e)
        {
            ShowData(null, true);
        }

        private void toolStripButtonPrev_Click(object sender, EventArgs e)
        {
            ShowData(FirstKey, false);
        }

        private void toolStripButtonNext_Click(object sender, EventArgs e)
        {
            ShowData(LastKey, true);
        }

        private void toolStripButtonEnd_Click(object sender, EventArgs e)
        {
            ShowData(null, false);
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public XTable Table
        {
            get { return table; }
            set
            {
                if (table != value)
                {
                    table = value;
                    PrepareColumns();
                }

                ShowData(FirstKey, true);
            }
        }
    }
}
