﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using STSdb.Data;
using System.Reflection;
using System.Diagnostics;

namespace GettingStarted
{
    public partial class MainForm : Form
    {
        private GettingStarted example = new GettingStarted(Path.Combine(Application.StartupPath, "Demo.stsdb"));

        public MainForm()
        {
            InitializeComponent();
        }

        private void Log(object obj)
        {
            string s = String.Format("{0:HH:mm:ss.fff}  {1}\r\n", DateTime.Now, obj);
            tbLog.AppendText(s);
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            example.OpenRepository();
            ShowScheme();
            TreeView_AfterSelect(TreeView, new TreeViewEventArgs(TreeView.SelectedNode));

            //load functions
            foreach (var method in typeof(GettingStarted).GetMethods().Where(x => x.IsPublic && x.ReturnType == typeof(Locator)))
                lbMethods.Items.Add(method);
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            example.CloseRepository();
        }

        private void ShowScheme()
        {
            foreach (var row in example.Repository.Scheme)
                TreeView.Nodes.BuildNode(row.Key.Items).Tag = row.Key;

            TreeView.ExpandAll();
        }

        private void btnCreateTable_Click(object sender, EventArgs e)
        {
            Locator locator = new Locator(tbTableName.Text);
            ulong count = ulong.Parse(tbTickCount.Text);

            Log(String.Format("Add {0} ticks in '{1}'...", count, locator));
            example.RecordCount = count;
            example.Append(locator);
            ulong records = example.GetRecordCount(locator);
            Log(String.Format("Done. {0} records available", records));

            ShowScheme();
            TreeView.SelectedNode = TreeView.Nodes.BuildNode(locator.Items);
            dataGridFrame1.Table = example.Repository.Scheme.OpenXTable(locator);
        }

        private void TreeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            btnExecute.Enabled = false;

            if (e.Node == null)
                return;

            Locator locator = e.Node.Tag as Locator;
            if (locator == null)
            {
                dataGridFrame1.Table = null;
                return;
            }

            XTable table = example.Repository.Scheme.OpenXTable(locator);
            btnExecute.Enabled = table.KeyType == typeof(ulong) && table.RecordType == typeof(Tick);
            dataGridFrame1.Table = table;
        }

        private void lbMethods_Format(object sender, ListControlConvertEventArgs e)
        {
            MethodInfo method = (MethodInfo)e.ListItem;
            e.Value = method.Name;
        }

        private void btnExecute_Click(object sender, EventArgs e)
        {
            MethodInfo method = lbMethods.SelectedItem as MethodInfo;
            if (method == null)
                return;

            Cursor = Cursors.WaitCursor;
            try
            {
                Locator locator = new Locator(tbTableName.Text);
                Locator dstLocator = (Locator)method.Invoke(example, new object[] { locator });

                ShowScheme();
                //TreeView.SelectedNode = TreeView.Nodes.BuildNode(dstLocator.Items);
            }
            finally
            {
                Cursor = Cursors.Arrow;
            }
        }
    }

    public static class TreeNodeCollectionExtensions
    {
        public static TreeNode BuildNode(this TreeNodeCollection self, params string[] path)
        {
            TreeNode node = null;
            for (int i = 0; i < path.Length; i++)
            {
                node = self.Find(path[i], false).FirstOrDefault();
                if (node == null)
                    node = self.Add(path[i], path[i]);

                self = node.Nodes;
            }

            return node;
        }
    }
}
